
# Copyright 2022 introt <introt@koti.fimnet.fi>

# based on original work (c) Jaap Karssenberg <jaap.karssenberg@gmail.com>

import tests

from tests.mainwindow import setUpMainWindow

from zim.notebook.page import Page, Path
from zim.gui.mainwindow import PageWindow, WindowBaseMixin
from zim.plugins.windowtitleeditor import (
		WindowTitleEditorPlugin,
		WindowTitleEditorExtension,
		READONLY)

class TestWindowTitleEditorExtension(tests.TestCase):

	def setUp(self):
		""" Successful setup indicates the monkey patching was successful... """
		self.plugin = WindowTitleEditorPlugin()
		self.window = setUpMainWindow(self.setUpNotebook())
		self.extension = WindowTitleEditorExtension(self.plugin, self.window)

	def testMonkeyPatch(self):
		""" ...but we test it anyway :) """
		self.assertEqual(
				self.window._update_window_title,
				self.extension.update_window_title,
				"Patching main window failed!")
		self.assertEqual(PageWindow.set_title.__module__,
				self.window._update_window_title.__module__,
				"Patching PageWindow failed!")

	def get_title(self):
		# used twice, just think of the convenience!
		return self.window.get_title()

	def testMainWindowApi(self):
		self.window.set_title('test')
		self.assertEqual(self.get_title(), 'test', "Can't get/set title!")

	def testMakeTitleWithCustomTemplate(self):
		n = self.window.notebook
		n.readonly = True
		p = self.window.page
		self.plugin.preferences['custom_format'] = ' $path;$page;$title;$source;' + \
				'$notebook;$folder$$Zim$ro'
		self.plugin.preferences['format'] = 'custom'
		self.assertEqual(
				self.get_title(),
				f'Test;Test;Test;{p.source_file};' + \
						f'testnotebook;{n.folder}$Zim{READONLY}'.strip(),
				"Templating broke!"
		)

	def testTeardown(self):
		self.extension.teardown()
		self.assertEqual(
				PageWindow.set_title,
				WindowBaseMixin.set_title,
				"Restoring PageWindow failed!"
		)
		self.assertEqual(
				self.window._update_window_title.__func__,
				self.window.__class__._update_window_title,
				"Restoring main window failed!"
		)



# coding=utf-8

import tests

from zim.plugins import PluginManager
from zim.plugins import indexed_fts

@tests.skipIf(
	indexed_fts.IndexedFTSPlugin.check_dependencies()[0] == False,
	"Indexed FTS plugin not available"
)
class TestIndexedFTS(tests.TestCase):

	def testIndexing(self):
		'''Check indexing of Indexed Full Text Search plugin'''
		plugin = PluginManager.load_plugin('indexed_fts')
		notebook = self.setUpNotebook(content=tests.FULL_NOTEBOOK)

		notebook.index.check_and_update()

		# Only pages with content will actually be indexed, so
		# after indexing some should be FTS-indexed.
		self.assertNotEqual(
			notebook.index._db.execute(
				"SELECT count(*) FROM keys_pages_fts WHERE page_id IS NOT NULL;"
			).fetchone()[0], 0
		)






# Copyright 2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>

import os
import sysconfig
import warnings

import tests
import zim.datetimetz as datetime


class TestDateTimeZ(tests.TestCase):

	# FIXME would be better to test correctness of results
	#       but first check functions do not give errors

	def setUp(self):
		with warnings.catch_warnings():
			warnings.simplefilter("ignore")
			try:
				import babel
			except ImportError:
				pass

	def runTest(self):
		# now()
		dt = datetime.now()
		s = dt.isoformat()
		self.assertTrue(isinstance(s, str) and len(s) > 0)

		s = dt.strftime("%z")
		self.assertTrue(isinstance(s, str) and len(s) > 0)

		s = dt.strftime("%Z")
		self.assertTrue(isinstance(s, str) and len(s) > 0)

		# strftime
		s = datetime.strftime('%a', dt)
		self.assertTrue(isinstance(s, str) and len(s) > 0)

		s = datetime.strftime('%%', dt)
		self.assertEqual(s, '%')

		# is not mingw? https://github.com/msys2/msys2.github.io/blob/source/web/docs/python.md?plain=1#L12
		if not (os.name == 'nt' and sysconfig.get_platform().startswith('mingw')):
			# Failed under msys python3.7.2
			s = datetime.strftime('%u', dt)
			self.assertTrue(isinstance(s, str) and len(s) > 0)

			s = datetime.strftime('%V', dt)
			self.assertTrue(isinstance(s, str) and len(s) > 0)

		s = datetime.strftime('%Y 道', dt)
		self.assertTrue(isinstance(s, str) and len(s) == 4 + 1 + 1)

		s = datetime.strftime('%Y ❗', dt)
		self.assertTrue(isinstance(s, str) and len(s) == 4 + 1 + 1)

		s = datetime.strftime('%Y 👨‍👩‍👧‍👦', dt)
		self.assertTrue(isinstance(s, str) and len(s) >= 4 + 1 + 1)

		s = datetime.strftime('%Y ƑÊẶṜ', dt)
		self.assertTrue(isinstance(s, str) and len(s) >= 4 + 1 + 4)

		# strfcal
		s = datetime.strfcal('%w', dt)
		self.assertTrue(isinstance(s, str) and len(s) > 0)

		s = datetime.strfcal('%W', dt)
		self.assertTrue(isinstance(s, str) and len(s) > 0)

		s = datetime.strfcal('%Y', dt)
		self.assertTrue(isinstance(s, str) and len(s) > 0)

		s = datetime.strfcal('%%', dt)
		self.assertEqual(s, '%')

		# weekcalendar
		year, week, weekday = datetime.weekcalendar(dt)
		self.assertTrue(isinstance(year, int) and 1900 < year and 3000 > year)
		self.assertTrue(isinstance(week, int) and 1 <= week and 53 >= week)
		self.assertTrue(isinstance(weekday, int) and 1 <= weekday and 7 >= weekday)

		# dates_for_week
		start, end = datetime.dates_for_week(year, week)
		self.assertTrue(isinstance(start, datetime.date))
		self.assertTrue(isinstance(end, datetime.date))
		self.assertTrue(start <= dt.date() and end >= dt.date())



# Copyright 2024 Jaap Karssenberg <jaap.karssenberg@gmail.com>

# TODO: there are more tests for find functionality in tests.pageview that could be merged here

import tests

from tests.pageview import TextBufferTestCaseMixin, Path, TextBuffer

from zim.gui.pageview.find import FindQuery, TextBufferFindMixin, PluginInsertedObjectFindMixin, \
	FIND_CASE_SENSITIVE, FIND_REGEX, FIND_WHOLE_WORD, FIND_HAS_MATCH, FIND_HAS_HIGHLIGHT

import re

from gi.repository import Gtk


class TestGtkTextBuffer(TextBufferFindMixin, Gtk.TextBuffer):

	def __init__(self):
		Gtk.TextBuffer.__init__(self)
		TextBufferFindMixin.__init__(self)


class TestFindQuery(tests.TestCase):

	def runTest(self):
		#i = 0
		for (string, flags, regex) in (
			('test*', 0, re.compile('test\\*', re.I)),
			('test*', FIND_CASE_SENSITIVE, re.compile('test\\*')),
			('test*', FIND_CASE_SENSITIVE | FIND_REGEX, re.compile('test*')),
			('test*', FIND_CASE_SENSITIVE | FIND_REGEX | FIND_WHOLE_WORD, re.compile('\\btest*\\b')),
			('test*', FIND_REGEX, re.compile('test*', re.I)),
			('test*', FIND_REGEX | FIND_WHOLE_WORD, re.compile('\\btest*\\b', re.I)),
			('test*', FIND_WHOLE_WORD, re.compile('\\btest\\*\\b', re.I)),
			('test*', FIND_WHOLE_WORD | FIND_CASE_SENSITIVE, re.compile('\\btest\\*\\b')),
		):
			#i += 1
			#print(">>", i, string, flags, regex)
			query = FindQuery(string, flags)
			self.assertEqual(query.string, string)
			self.assertEqual(query.flags, flags)
			self.assertEqual(query.regex, regex)

			query1 = FindQuery(string, flags)
			self.assertEqual(query1, query)


class TestFindWithGtkTextBuffer(tests.TestCase):

	TEXT ='''\
Some text here
Word words words

Some more text in this textbuffer

TEXT test text
'''

	FORWARD_MATCHES_FROM_LINE_TWO = (
			(3, 10, 'text'),
			# Ignore partial match "textbuffer"
			(5, 0, 'TEXT'), # Case insensitive match
			(5, 10, 'text'),
			# Wrap around
			(0, 5, 'text'),
			(3, 10, 'text') # back to start
		)

	BACKWARD_MATCHES_FROM_LINE_TWO = (
			(0, 5, 'text'),
			# Wrap around
			(5, 10, 'text'),
			(5, 0, 'TEXT'), # Case insensitive match
			# Ignore partial match "textbuffer"
			(3, 10, 'text'),
			(0, 5, 'text') # back to start
		)

	ALL_MATCHES_FOR_HIGHLIGHT = [(0, 5), (3, 10), (5, 0), (5, 10)]

	TEXT_REPLACE_ONE = '''\
Some text here
Word words words

Some more mytext in this textbuffer

TEXT test text
'''

	TEXT_REPLACE_ALL = '''\
Some mytext here
Word words words

Some more mytext in this textbuffer

myTEXT test mytext
'''

	def setUp(self):
		self.buffer = TestGtkTextBuffer()
		self.buffer.set_text(self.TEXT)

	def testFindNext(self):
		query = FindQuery('text', FIND_WHOLE_WORD)
		self.buffer.place_cursor(self.buffer.get_iter_at_line(2))
		for line, offset, text in self.FORWARD_MATCHES_FROM_LINE_TWO:
			self.assertTrue(self.buffer.find_next(query))
			self.assertMatchPosition(line, offset, text)

	def testFindNextMatchesAtCursor(self):
		# Behavior for match at cursor depends on whether it was highlighted already or not
		query = FindQuery('text', FIND_WHOLE_WORD)
		self.buffer.place_cursor(self.buffer.get_iter_at_line(2))
		match_pos_one = self.FORWARD_MATCHES_FROM_LINE_TWO[0]
		match_pos_two = self.FORWARD_MATCHES_FROM_LINE_TWO[1]

		self.assertTrue(self.buffer.find_next(query))
		self.assertMatchPosition(*match_pos_one)
		self.buffer.find_clear() # Reset matching state
		self.assertTrue(self.buffer.find_next(query))
		self.assertMatchPosition(*match_pos_one) # stay at cursor
		# no reset
		self.assertTrue(self.buffer.find_next(query))
		self.assertMatchPosition(*match_pos_two) # move forward

	def testFindPrevious(self):
		query = FindQuery('text', FIND_WHOLE_WORD)
		self.buffer.place_cursor(self.buffer.get_iter_at_line(2))
		for line, offset, text in self.BACKWARD_MATCHES_FROM_LINE_TWO:
			self.assertTrue(self.buffer.find_previous(query))
			self.assertMatchPosition(line, offset, text)

	def assertMatchPosition(self, line, offset, text):
		# Check cursor position
		cursor_iter = self.buffer.get_iter_at_mark(self.buffer.get_insert())
		cursor_pos = (cursor_iter.get_line(), cursor_iter.get_line_offset())
		self.assertEqual(cursor_pos, (line, offset))

		if text is None:
			return # Skip checks below for object matches

		# Check selection applied
		end_iter = cursor_iter.copy()
		start, end = self.buffer.get_selection_bounds()
		self.assertTrue(start.equal(cursor_iter))
		self.assertEqual(start.get_text(end), text)

		# Check match tag applied
		start = self.buffer.get_start_iter()
		start.forward_to_tag_toggle(self.buffer._find_match_tag)
		end = start.copy()
		end.forward_to_tag_toggle(self.buffer._find_match_tag)
		self.assertEqual(start.get_text(end), text)
		self.assertTrue(start.equal(cursor_iter))
		self.assertEqual(start.get_text(end), text)
		end.forward_to_tag_toggle(self.buffer._find_match_tag)
		self.assertFalse(end.forward_to_tag_toggle(self.buffer._find_match_tag)) # no other tag found before end of buffer

	def testHighlightAll(self):
		query = FindQuery('text', FIND_WHOLE_WORD)
		self.assertHighlightPositions([], 'text')
		self.buffer.find_highlight_all(query)
		self.assertHighlightPositions(self.ALL_MATCHES_FOR_HIGHLIGHT, 'text')
		self.buffer.find_clear()
		self.assertHighlightPositions([], 'text')

	def testFindNextPreviousClearHighlight(self):
		# Test the implied statefull behavior
		query = FindQuery('text', FIND_WHOLE_WORD)
		self.buffer.find_highlight_all(query)
		self.assertHighlightPositions(self.ALL_MATCHES_FOR_HIGHLIGHT, 'text')
		self.buffer.find_next(query)
		self.assertHighlightPositions(self.ALL_MATCHES_FOR_HIGHLIGHT, 'text')
		self.buffer.find_previous(query)
		self.assertHighlightPositions(self.ALL_MATCHES_FOR_HIGHLIGHT, 'text')

		new_query = FindQuery('word')
		self.buffer.find_next(new_query)
		self.assertHighlightPositions([], 'text')

		self.buffer.find_highlight_all(query)
		self.assertHighlightPositions(self.ALL_MATCHES_FOR_HIGHLIGHT, 'text')
		self.buffer.find_previous(new_query)
		self.assertHighlightPositions([], 'text')

	def assertHighlightPositions(self, wanted, text):
		iter = self.buffer.get_start_iter()
		matches = []
		while iter.forward_to_tag_toggle(self.buffer._find_highlight_tag):
			matches.append((iter.get_line(), iter.get_line_offset()))
			start = iter.copy()
			iter.forward_to_tag_toggle(self.buffer._find_highlight_tag)
			self.assertEqual(start.get_text(iter).lower(), text)
		self.assertEqual(matches, wanted)

	def testReplaceAtCursor(self):
		query = FindQuery('text', FIND_WHOLE_WORD)
		self.buffer.place_cursor(self.buffer.get_iter_at_line(2))
		self.buffer.find_replace_at_cursor(query, 'mytext')
		start, end = self.buffer.get_bounds()
		self.assertEqual(start.get_text(end), self.TEXT) # no change, since no match at cursor
		self.assertTrue(self.buffer.find_next(query))
		self.assertTrue(self.buffer.find_replace_at_cursor(query, 'mytext'))
		start, end = self.buffer.get_bounds()
		self.assertEqual(start.get_text(end), self.TEXT_REPLACE_ONE)

	def testReplaceAll(self):
		# Added FIND_REGEX to test regex replacement
		query = FindQuery('(text)', FIND_WHOLE_WORD | FIND_REGEX)
		self.assertTrue(self.buffer.find_replace_all(query, 'my\\1'))
		start, end = self.buffer.get_bounds()
		self.assertEqual(start.get_text(end), self.TEXT_REPLACE_ALL)


def _get_text_matches(text, query):
	matches = []
	for i, line in enumerate(text.splitlines()):
		for m in query.regex.finditer(line):
			j = m.start()
			matches.append((i, j, m.group()))
	return matches


def _find_object_matches(buffer, query):
	for iter, anchor in buffer.list_objectanchors():
		if hasattr(anchor, 'objecttype'):
			attrib, data = anchor.objecttype.data_from_model(anchor.objectmodel)
			if data and query.regex.search(data):
				yield iter.get_line(), iter.get_line_offset(), None


class TestFindWithFormattedTextBuffer(TestFindWithGtkTextBuffer):
	# Extension of test case above testing in a buffer with formatting applied and some objects

	# These values are generated in setup below
	TEXT = None
	FORWARD_MATCHES_FROM_LINE_TWO = None
	BACKWARD_MATCHES_FROM_LINE_TWO = None
	ALL_MATCHES_FOR_HIGHLIGHT = None
	TEXT_REPLACE_ONE = None
	TEXT_REPLACE_ALL = None

	def setUp(self):
		notebook = self.setUpNotebook()
		page = notebook.get_page(Path('Test'))
		self.buffer = TextBuffer(notebook, page)
		tree = tests.new_parsetree()
		with tests.LoggingFilter('zim.gui.pageview'):
			self.buffer.set_parsetree(tree)

		start, end = self.buffer.get_bounds()
		self.TEXT = start.get_text(end)
		query = FindQuery('text', FIND_WHOLE_WORD)
		text_matches = _get_text_matches(self.TEXT, query)
		assert len(text_matches) == 3, 'Should be 3 text matches'
		self.ALL_MATCHES_FOR_HIGHLIGHT = [m[0:2] for m in text_matches] # Text matches only

		object_matches = list(_find_object_matches(self.buffer, query))
		assert len(object_matches) == 1, 'Should be 1 object match'
		self.FORWARD_MATCHES_FROM_LINE_TWO = tuple(sorted(text_matches + object_matches))
		rev_object_matches = [(l, c+1, m) for l, c, m in object_matches] # cursor *behind* object
		self.BACKWARD_MATCHES_FROM_LINE_TWO = tuple(reversed(sorted(text_matches + rev_object_matches)))

		self.TEXT_REPLACE_ONE = re.sub('\\btext\\b', 'mytext', self.TEXT, count=1)
		self.TEXT_REPLACE_ALL = re.sub('\\btext\\b', 'mytext', self.TEXT)


class MockSimplePluginObjectType():

	is_inline = True

	def data_from_model(self, model):
		return {}, model.data


class MockSimplePluginObjectModel():

	def __init__(self, data):
		self.data = data


class TestFindWithSimplePluginObject(tests.TestCase):

	DATA = 'Foo bar baz'

	def setUp(self):
		from zim.gui.pageview.objectanchors import PluginInsertedObjectAnchor

		self.anchor = PluginInsertedObjectAnchor(MockSimplePluginObjectType(), MockSimplePluginObjectModel(self.DATA))
		self.query = FindQuery('bar')

	def testFindNext(self):
		self._test_next_prev(self.anchor.find_next)

	def testFindPrevious(self):
		self._test_next_prev(self.anchor.find_previous)

	def _test_next_prev(self, method):
		self.assertTrue(method(self.query))
		self.assertTrue(self.anchor._find_match_highlight_state == FIND_HAS_MATCH)
		self.assertFalse(method(self.query)) # no "next" match if already highlighting current match
		self.assertTrue(self.anchor._find_match_highlight_state == 0)
		self.assertTrue(method(self.query))
		self.assertFalse(method(self.query)) # no "next" match if already highlighting current match

		self.assertTrue(method(self.query))
		self.assertTrue(self.anchor._find_match_highlight_state == FIND_HAS_MATCH)
		self.anchor.find_clear()
		self.assertTrue(self.anchor._find_match_highlight_state == 0)

		fail_query = FindQuery('No match!')
		self.assertFalse(method(fail_query))
		self.assertFalse(method(fail_query))

	def testHighlightAll(self):
		self.anchor.find_highlight_all(self.query)
		self.assertTrue(self.anchor._find_match_highlight_state == FIND_HAS_HIGHLIGHT)
		self.anchor.find_clear()
		self.assertTrue(self.anchor._find_match_highlight_state == 0)

	def testFindNextPreviousClearHighlight(self):
		# Test the implied statefull behavior
		self.anchor.find_highlight_all(self.query)
		self.assertTrue(self.anchor._find_match_highlight_state == FIND_HAS_HIGHLIGHT)
		self.assertTrue(self.anchor.find_next(self.query))
		self.assertTrue(self.anchor._find_match_highlight_state == FIND_HAS_HIGHLIGHT | FIND_HAS_MATCH)
		self.assertFalse(self.anchor.find_next(self.query))
		self.assertTrue(self.anchor._find_match_highlight_state == FIND_HAS_HIGHLIGHT)

		new_query = FindQuery('baz')
		self.assertTrue(self.anchor.find_next(new_query))
		self.assertTrue(self.anchor._find_match_highlight_state == FIND_HAS_MATCH)

	def testReplaceAtCursor(self):
		self.assertFalse(self.anchor.find_replace_at_cursor(self.query, 'foo')) # not supported

	def testReplaceAll(self):
		self.assertFalse(self.anchor.find_replace_all(self.query, 'foo')) # not supported


class MockPluginObjectModelWithOverload():

	def __init__(self, data):
		self.data = data

	def find_simple_match(self, query):
		return query.regex.search(self.data) is not None


class TestFindWithBackwardImageGeneratorPluginObject(TestFindWithSimplePluginObject):

	# Testing overload interface used for image generators

	def setUp(self):
		from zim.gui.pageview.objectanchors import PluginInsertedObjectAnchor

		self.anchor = PluginInsertedObjectAnchor(MockSimplePluginObjectType(), MockPluginObjectModelWithOverload(self.DATA))
		self.query = FindQuery('bar')


#class TestFindWithNestedTextBufferPluginObject(tests.TestCase):

#	def setUp(self):
#		pass

#	def testFindNext(self):
#		pass

#	def testFindPrevious(self):
#		pass

#	def testHighlightAll(self):
#		pass # include clear

#	def testFindNextPreviousClearHighlight(self):
#		# Test the implied statefull behavior
#		pass

#	def testReplaceAtCursor(self):
#		pass

#	def testReplaceAll(self):
#		pass


class TestFindOptionsWithTextBuffer(tests.TestCase, TextBufferTestCaseMixin):
		# Older set of tests, kept as "monkey testing" of the interface

	def runTest(self):
		notebook = self.setUpNotebook()
		page = notebook.get_page(Path('Test'))
		buffer = TextBuffer(notebook, page)
		buffer.set_text('''\
FOO FooBar FOOBAR
FooBaz Foo Bar
foo Bar Baz Foo
''')
		buffer.place_cursor(buffer.get_start_iter())

		# Check normal usage, case-insensitive
		for text in ('f', 'fo', 'foo', 'fo', 'f', 'F', 'Fo', 'Foo'):
			buffer.find_next(FindQuery(text))
			self.assertSelection(buffer, 0, 0, text.upper())
			buffer.find_clear() # prevent skip to next occurence

		buffer.find_next(FindQuery('Grr'))
		self.assertCursorPosition(buffer, 0, 0)

		buffer.find_next(FindQuery('Foob'))
		self.assertSelection(buffer, 0, 4, 'FooB')

		for line, offset, text in (
			(0, 11, 'FOOB'),
			(1, 0, 'FooB'),
			(0, 4, 'FooB'),
		):
			buffer.find_next(FindQuery('Foob'))
			self.assertSelection(buffer, line, offset, text)

		for line, offset, text in (
			(1, 0, 'FooB'),
			(0, 11, 'FOOB'),
			(0, 4, 'FooB'),
		):
			buffer.find_previous(FindQuery('Foob'))
			self.assertSelection(buffer, line, offset, text)

		# Case sensitive
		buffer.find_clear() # reset state
		buffer.find_next(FindQuery('Foo', FIND_CASE_SENSITIVE))
		self.assertSelection(buffer, 0, 4, 'Foo')

		for line, offset, text in (
			(1, 0, 'Foo'),
			(1, 7, 'Foo'),
			(2, 12, 'Foo'),
			(0, 4, 'Foo'),
		):
			buffer.find_next(FindQuery('Foo', FIND_CASE_SENSITIVE))
			self.assertSelection(buffer, line, offset, text)

		# Whole word
		buffer.find_clear() # reset state
		buffer.find_next(FindQuery('Foo', FIND_WHOLE_WORD))
		self.assertSelection(buffer, 1, 7, 'Foo')

		for line, offset, text in (
			(2, 0, 'foo'),
			(2, 12, 'Foo'),
			(0, 0, 'FOO'),
			(1, 7, 'Foo'),
		):
			buffer.find_next(FindQuery('Foo', FIND_WHOLE_WORD))
			self.assertSelection(buffer, line, offset, text)

		# Regular expression
		buffer.find_clear() # reset state
		query = FindQuery(r'Foo\s*Bar', FIND_REGEX | FIND_CASE_SENSITIVE)
		buffer.find_next(query)
		self.assertSelection(buffer, 1, 7, 'Foo Bar')
		buffer.find_next(query)
		self.assertSelection(buffer, 0, 4, 'FooBar')



#!/usr/bin/python3

import base64
import sys

from zim.gui.applications import get_mimetype
from zim.newfs import LocalFile


def data_uri(file):
	mimetype = get_mimetype(file)
	data64 = base64.b64encode(file.read_binary()).decode('utf-8')
	return f'data:{mimetype};base64,{data64}'


def main():
	file = LocalFile(sys.argv[1])
	assert file.exists(), f'File \'{file}\' does not exist'
	print(data_uri(file))


if __name__ == '__main__':
	main()




# Copyright 2009-2024 Jaap Karssenberg <jaap.karssenberg@gmail.com>

import tests

import zim.datetimetz as datetime

from zim.parse import convert_space_to_tab, fix_unicode_whitespace
from zim.parse.builder import BuilderTextBuffer
from zim.parse.encode import \
	escape_string, unescape_string, split_escaped_string, \
	url_decode, url_encode, \
	URL_ENCODE_READABLE, URL_ENCODE_PATH, URL_ENCODE_DATA
from zim.parse.links import *
from zim.parse.dates import *
from zim.parse.dates import old_parse_date
from zim.parse.regexparser import *
from zim.parse.simpletree import *
from zim.parse.tokenlist import EndOfTokenListError, TokenBuilder, TokenParser, \
	collect_until_end_token, filter_token, reverseTopLevelLists, \
	testTokenStream, tokens_to_text, topLevelLists
from zim.formats import ParseTreeBuilder, PARAGRAPH


class TestEscapeStringFunctions(tests.TestCase):

	def testEscapeString(self):
		for raw, escaped in (
			('Newline \n', 'Newline \\n'),
			('Tab \t', 'Tab \\t'),
			('Special char |', 'Special char \\|'),
			('Backslash \\', 'Backslash \\\\'),
			('Backslashed special char \\|', 'Backslashed special char \\\\\\|'),
			('Not a newline \\n', 'Not a newline \\\\n'),
		):
			self.assertEqual(escape_string(raw, chars='|'), escaped)
			self.assertEqual(unescape_string(escaped), raw)

	def testSplitEscapedString(self):
		for string, parts in (
			('Part A|Part B|Part C', ['Part A', 'Part B', 'Part C']),
			('Part A\\| with pipe|Part B|Part C', ['Part A\\| with pipe', 'Part B', 'Part C']),
			('Part A\\\\\\| with multiple backslash|Part B|Part C', ['Part A\\\\\\| with multiple backslash', 'Part B', 'Part C']),
			('Part A with backslash\\\\|Part B|Part C', ['Part A with backslash\\\\', 'Part B', 'Part C']),
			('No agressive strip \\', ['No agressive strip \\'])
		):
			self.assertEqual(split_escaped_string(string, '|'), parts)


class TestURLEncode(tests.TestCase):

	def testURLEncoding(self):
		'''Test encoding and decoding urls'''
		for url, readable in (
			('file:///foo/file%25%20%5D', 'file:///foo/file%25 %5D'),
			('http://foo/bar%20monkey%E2%80%99s', 'http://foo/bar monkey\u2019s'), # Multibyte unicode char

			# from bug report lp:545712
			('http://www.moneydj.com/e/newage/JAVA%B4%FA%B8%D5%B0%CF.htm',
				'http://www.moneydj.com/e/newage/JAVA%B4%FA%B8%D5%B0%CF.htm'),
			('http://www.moneydj.com/e/newage/JAVA%20%B4%FA%B8%D5%B0%CF.htm',
				'http://www.moneydj.com/e/newage/JAVA %B4%FA%B8%D5%B0%CF.htm'),
		):
			self.assertEqual(url_decode(url, mode=URL_ENCODE_READABLE), readable)
			self.assertEqual(url_decode(readable, mode=URL_ENCODE_READABLE), readable)
			self.assertEqual(url_encode(url, mode=URL_ENCODE_READABLE), url)
			self.assertEqual(url_encode(readable, mode=URL_ENCODE_READABLE), url)

		for path, encoded in (
			('/foo/file% ]', '/foo/file%25%20%5D'),
			('/foo/bar monkey\u2019s', '/foo/bar%20monkey%E2%80%99s'),
		):
			self.assertEqual(url_encode(path, mode=URL_ENCODE_PATH), encoded)
			self.assertEqual(url_decode(encoded, mode=URL_ENCODE_PATH), path)

		self.assertEqual(url_encode('foo?bar/baz', mode=URL_ENCODE_DATA), 'foo%3Fbar%2Fbaz')
		self.assertEqual(url_decode('foo%3Fbar%2Fbaz', mode=URL_ENCODE_DATA), 'foo?bar/baz')
		# from bug report lp:545712
		self.assertEqual(url_decode('%B4%FA%B8%D5%B0%CF', mode=URL_ENCODE_DATA), '%B4%FA%B8%D5%B0%CF')

		## test round trip for utf-8
		data = '\u0421\u0430\u0439'
		encoded = url_encode(data)
		decoded = url_decode(data)
		#~ print("DATA, ENCODED, DECODED:", (data, encoded, decoded))
		self.assertEqual(decoded, data)
		self.assertEqual(url_decode(encoded), data)
		self.assertEqual(url_encode(decoded), encoded)


class TestParseLinks(tests.TestCase):

	def testLinkType(self):
		'''Test link_type()'''
		for href, type in (
			('zim+file://foo/bar?dus.txt', 'notebook'),
			('file:///foo/bar', 'file'),
			('file://foo/bar', 'file'),
			('file://localhost/foo/bar', 'file'),
			('file:/foo/bar', 'file'),
			('http://foo/bar', 'http'),
			('http://192.168.168.100', 'http'),
			('file+ssh://foo/bar', 'file+ssh'),
			('mailto:foo@bar.com', 'mailto'),
			('mailto:foo.com', 'page'),
			('foo@bar.com', 'mailto'),
			('mailto:foo//bar@bar.com', 'mailto'), # is this a valid mailto uri ?
			('mid:foo@bar.org', 'mid'),
			('cid:foo@bar.org', 'cid'),
			('./foo/bar', 'file'),
			('/foo/bar', 'file'),
			('~/foo', 'file'),
			('C:\\foo', 'file'),
			('wp?foo', 'interwiki'),
			('http://foo?bar', 'http'),
			('\\\\host\\foo\\bar', 'smb'),
			('foo', 'page'),
			('foo:bar', 'page'),
		):
			# print('>>', href)
			self.assertEqual(link_type(href), type)


class TestGFMAutolinks(tests.TestCase):
	# See https://github.github.com/gfm/#autolinks-extension-

	examples = (
		# Basic match
		('www.commonmark.org', True, None),
		('www.commonmark.org/help', True, None),
		('http://commonmark.org', True, None),
		('http://commonmark.org/help', True, None),
		('commonmark.org', False, None),
		('commonmark.org/help', False, None),


		# No "_" in last two parts domain
		('www.common_mark.org', False, None),
		('www.commonmark.org_help', False, None),
		('www.test_123.commonmark.org', True, None),

		# Trailing punctuation
		('www.commonmark.org/a.b.', True, '.'),
		('www.commonmark.org.', True, '.'),
		('www.commonmark.org?', True, '?'),

		# Trailing ")"
		('www.google.com/search?q=Markup+(business)', True, None),
		('www.google.com/search?q=Markup+(business))', True, ')'),
		('www.google.com/search?q=Markup+(business)))', True, '))'),
		('www.google.com/search?q=(business))+ok', True, None),

		# Trailing entity reference
		('www.google.com/search?q=commonmark&hl=en', True, None),
		('www.google.com/search?q=commonmark&hl;', True, '&hl;'),

		# A "<" always breaks the link
		('www.commonmark.org/he<lp', True, '<lp'),

		# Email
		('foo@bar.baz', True, None),
		('hello@mail+xyz.example', False, None),
		('hello+xyz@mail.example', True, None),
		('a.b-c_d@a.b', True, None),
		('a.b-c_d@a.b.', True, '.'),
		('a.b-c_d@a.b-', False, None),
		('a.b-c_d@a.b_', False, None),
		('@tag', False, None),

		# Examples from bug tracker
		('https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.10/#container-core-v1-', True, None),
		('https://da.sharelatex.com/templates/books/springer\'s-monograph-type-svm', True, None),
		('https://en.wikipedia.org/wiki/80/20_(framing_system)', True, None),
		('https://bugs.kde.org/buglist.cgi?resolution=---', True, None),
		#('https://vimhelp.org/options.txt.html#\'iskeyword\'', True, None),
		#	-> this example is overruled by new behavior
		('https://example.com/foo]', True, None),

		# Zim extensions
		('https://localhost', True, None),
		('https://localhost/path', True, None),
		('file:///home/foo', True, None),
		('file://home/foo', True, None),
		('file:/home/foo', True, None),
		('foo://bar', True, None),
	)

	def testFunctions(self):
		for input, input_is_url, tail in self.examples:
			if input_is_url:
				if tail:
					self.assertEqual(match_url_link(input), input[:-len(tail)])
					self.assertFalse(is_url_link(input))
				else:
					self.assertEqual(match_url_link(input), input)
					self.assertTrue(is_url_link(input))
			else:
				self.assertEqual(match_url_link(input), None)
				self.assertFalse(is_url_link(input))


class TestParseDates(tests.TestCase):

	def testParseDate(self):
		date = datetime.date(2017, 3, 27)
		for text in (
			'2017-03-27', '2017-03',
			'2017-W13', '2017-W13-1',
			'2017W13', '2017W13-1',
			'2017w13', '2017w13-1',
			'W1713', 'W1713-1', 'W1713.1',
			'Wk1713', 'Wk1713-1', 'Wk1713.1',
			'wk1713', 'wk1713-1', 'wk1713.1',
		):
			m = date_re.match(text)
			self.assertIsNotNone(m, 'Failed to match: %s' % text)
			self.assertEqual(m.group(0), text)
			obj = parse_date(m.group(0))
			self.assertIsInstance(obj, (Day, Week, Month))
			self.assertTrue(obj.first_day <= date <= obj.last_day)

		for text in (
			'foo', '123foo', '2017-03-270',
			'20170317', '17-03-27', '17-03'
			'17W', '2017W131', '2017-W131'
		):
			m = date_re.match(text)
			if m:
				print('>>', m.group(0))
			self.assertIsNone(m, 'Did unexpectedly match: %s' % text)

	def testWeekNumber(self):
		self.assertEqual(
			Day(2017, 3, 27),
			Day.new_from_weeknumber(2017, 13, 1)
		)
		self.assertEqual(
			Day(2017, 3, 27).weekformat(),
			('2017-W13-1')
		)
		self.assertEqual(
			Day.new_from_weeknumber(2017, 13, 7),
			Day.new_from_weeknumber(2017, 14, 0)
		)

	def testOldParseDate(self):
		'''Test parsing dates'''
		from datetime import date
		today = date.today()
		year = today.year
		if today.month > 6:
			year += 1 # Starting July next year January is closer
		self.assertEqual(old_parse_date('1/1'), (year, 1, 1))
		self.assertEqual(old_parse_date('1-1'), (year, 1, 1))
		self.assertEqual(old_parse_date('1:1'), (year, 1, 1))
		self.assertEqual(old_parse_date('11/11/99'), (1999, 11, 11))
		self.assertEqual(old_parse_date('11/11/11'), (2011, 11, 11))
		self.assertEqual(old_parse_date('1/11/2001'), (2001, 11, 1))
		self.assertEqual(old_parse_date('1-11-2001'), (2001, 11, 1))
		self.assertEqual(old_parse_date('1:11:2001'), (2001, 11, 1))
		self.assertEqual(old_parse_date('2001/11/1'), (2001, 11, 1))


class TestSimpleTreeBuilder(tests.TestCase):

	def runTest(self):
		E = SimpleTreeElement

		builder = SimpleTreeBuilder()

		builder.start('root', {})
		builder.text('foo')
		builder.text('bar')
		builder.append('dus', {}, 'ja')
		builder.text('foo')
		builder.text('bar')
		builder.append('br', {})
		builder.text('foo')
		builder.text('bar')
		builder.end('root')

		root = builder.get_root()
		self.assertEqual(root,
			E('root', {}, [
					'foo', 'bar',
					E('dus', {}, ['ja']),
					'foo', 'bar',
					E('br', {}, []),
					'foo', 'bar',
				]
			)
		)


		realbuilder = SimpleTreeBuilder()
		builder = BuilderTextBuffer(realbuilder)

		builder.start('root', None)
		builder.text('foo')
		builder.text('bar')
		builder.append('bold', {'level': 3}, 'ja')
		builder.text('foo')
		builder.text('bar')
		builder.append('br', None)
		builder.text('foo')
		builder.text('bar')
		builder.end('root')

		root = realbuilder.get_root()
		self.assertEqual(root,
				E('root', None, [
					'foobar',
					E('bold', {'level': 3}, ['ja']),
					'foobar',
					E('br', None, []),
					'foobar',
				]
			)
		)


class TestTokensToSimpleTree(tests.TestCase):

	def setUp(self):
		E = SimpleTreeElement
		self.elt = E('root', None, [
				'foobar',
				E('bold', {'level': 3}, ['ja']),
				'foobar',
				E('br', None, []),
				'foobar',
			]
		)
		self.tokens = [
			('root', None),
				(TEXT, 'foobar'),
				('bold', {'level': 3}), (TEXT, 'ja'), (END, 'bold'),
				(TEXT, 'foobar'),
				('br', None), (END, 'br'),
				(TEXT, 'foobar'),
			(END, 'root')
		]

	def testSimpleTreeToTokens(self):
		self.assertEqual(simpletree_to_tokens(self.elt), self.tokens)

	def testSimpleTreeToTokensWithParserBuilder(self):
		builder = TokenBuilder()
		SimpleTreeParser(builder).parse(self.elt)
		self.assertEqual(builder.tokens, self.tokens)

	def testTokensToSimpleTree(self):
		self.assertEqual(tokens_to_simpletree(self.tokens), self.elt)

	def testTokensToSimpleTreeWithParserBuilder(self):
		builder = SimpleTreeBuilder()
		TokenParser(builder).parse(self.tokens)
		self.assertAlmostEqual(builder.get_root(), self.elt)


class TestBuilderTextBuffer(tests.TestCase):

	def runTest(self):
		builder = SimpleTreeBuilder()
		buffer = BuilderTextBuffer(builder)

		buffer.start('FOO')
		buffer.text('aaa\n')
		buffer.text('bbb\n')
		buffer.text('ccc\n')
		self.assertEqual(buffer.get_text(), 'aaa\nbbb\nccc\n')

		buffer.append('BAR')
		self.assertEqual(buffer.get_text(), '')

		buffer.text('qqq\n')
		self.assertEqual(buffer.get_text(), 'qqq\n')
		buffer.clear_text()

		buffer.text('qqq\n')
		self.assertEqual(buffer.get_text(), 'qqq\n')
		buffer.set_text('ddd\n')
		self.assertEqual(buffer.get_text(), 'ddd\n')

		buffer.text('')
		buffer.text('eee')
		buffer.end('FOO')

		E = SimpleTreeElement
		self.assertEqual(builder.get_root(), 
			E('FOO', None, [
				'aaa\nbbb\nccc\n',
				E('BAR', None, []),
				'ddd\neee',
			])
		)



class TestParser(tests.TestCase):

	def testFixUnicode(self):
		self.assertEqual(fix_unicode_whitespace('foo\u2028bar\u2029check\n'), 'foo\nbar check\n')

	def testConvertSpaceToTab(self):
		self.assertEqual(convert_space_to_tab('    foo\n\t     bar\n'), '\tfoo\n\t\t bar\n')

	def testGetLineCount(self):
		text = 'foo\nbar\nbaz\n'
		for offset, wanted in (
			(0, (1, 0)),
			(3, (1, 3)),
			(4, (2, 0)),
			(8, (3, 0)),
			(9, (3, 1)),
		):
			line = get_line_count(text, offset)
			self.assertEqual(line, wanted)

	## TODO -- Parser test cases ##


class TestTokenParser(tests.TestCase):

	def walk(self, tree, builder):
		# Iter tree without toplevellist
		for t in tree._get_tokens(tree._etree.getroot()):
			if t[0] == TEXT:
				builder.text(t[1])
			elif t[0] == END:
				builder.end(t[1])
			else:
				builder.start(t[0], t[1])

	def testRoundtrip(self):
		tree = tests.new_parsetree()
		#~ print tree
		tb = TokenBuilder()
		self.walk(tree, tb)
		tokens = tb.tokens
		#~ import pprint; pprint.pprint(tokens)

		testTokenStream(tokens)

		builder = ParseTreeBuilder()
		TokenParser(builder).parse(tokens)
		newtree = builder.get_parsetree()

		self.assertEqual(tree.tostring(), newtree.tostring())


	def testTopLevelLists(self):
		tree = tests.new_parsetree()
		tb = TokenBuilder()
		self.walk(tree, tb)
		tokens = tb._tokens # using raw tokens

		newtokens = topLevelLists(tokens)
		testTokenStream(newtokens)
		revtokens = reverseTopLevelLists(newtokens)

		def correct_none_attrib(t):
			if t[0] == PARAGRAPH and not t[1]:
				return (PARAGRAPH, {})
			else:
				return t

		revtokens = list(map(correct_none_attrib, revtokens))

		self.assertEqual(revtokens, tokens)


class TestFunctions(tests.TestCase):

	def testCollectTokens(self):
		# simple
		self.assertEqual(
			collect_until_end_token(
				[('B', {}), ('T', ''), (END, 'B'), (END, 'A'), ('T', '')],
				'A'
			),
				[('B', {}), ('T', ''), (END, 'B')]
		)
		# nested
		self.assertEqual(
			collect_until_end_token(
				[('A', {}), ('T', ''), (END, 'A'), (END, 'A'), ('T', '')],
				'A'
			),
				[('A', {}), ('T', ''), (END, 'A')]
		)
		# error case: no closing tag found
		with self.assertRaises(EndOfTokenListError):
			collect_until_end_token(
				[('B', {}), ('T', ''), (END, 'B'), ('T', '')],
				'A'
			)

	def testFilterTokens(self):
		self.assertEqual(
			list(filter_token(
				[('T', 'pre'), ('A', {}), ('T', 'inner'), (END, 'A'), ('T', 'post')],
				'A'
			)),
			[('T', 'pre'), ('T', 'post')]
		)
		self.assertEqual(
			list(filter_token(
				[('T', 'pre'), ('A', {}), ('A', {}), ('T', 'inner'), (END, 'A'), (END, 'A'), ('T', 'post')],
				'A'
			)),
			[('T', 'pre'), ('T', 'post')]
		)

	def testTokensToText(self):
		self.assertEqual(
			tokens_to_text([('B', {}), ('T', 'Foo'), (END, 'B'), ('T', 'Bar')]),
			'FooBar'
		)



#!/usr/bin/env python3

import os
import sys
import shutil
import subprocess
import glob


# Check if we run the correct python version
REQUIRED_MINIMUM_PYTHON_VERSION = (3, 6)
USED_PYTHON_VERSION = sys.version_info
if USED_PYTHON_VERSION < REQUIRED_MINIMUM_PYTHON_VERSION:
	error_message = 'zim needs python >= {major}.{minor}'.format(
		major=REQUIRED_MINIMUM_PYTHON_VERSION[0],
		minor=REQUIRED_MINIMUM_PYTHON_VERSION[1],
	)
	sys.stderr.write(error_message)
	sys.exit(1)

try:
	import py2exe
except ImportError:
	py2exe = None

from setuptools import setup, Command
from setuptools.command.sdist import sdist as sdist_class
from setuptools.command.build import build as build_class
from setuptools.command.install import install as install_class


from zim import __version__, __url__

import msgfmt # also distributed with zim
import makeman # helper script


# Some constants

PO_FOLDER = 'translations'
LOCALE_FOLDER = 'locale'


# Helper routines

# Function copied from `distutils.dep_util` to avoid complicated import when switching to setuptools
def newer(source, target):
    """Return true if 'source' exists and is more recently modified than
    'target', or if 'source' exists and 'target' doesn't.  Return false if
    both exist and 'target' is the same age or younger than 'source'.
    Raise DistutilsFileError if 'source' does not exist.
    """
    if not os.path.exists(source):
        raise DistutilsFileError("file '%s' does not exist" % os.path.abspath(source))
    if not os.path.exists(target):
        return 1

    from stat import ST_MTIME

    mtime1 = os.stat(source)[ST_MTIME]
    mtime2 = os.stat(target)[ST_MTIME]

    return mtime1 > mtime2


def find_scripts_in_build_dir():
	scripts = glob.glob('./build/scripts-*/*.py')
	assert len(scripts) > 0
	return scripts
	

def collect_packages():
	# Search for python packages below zim/
	packages = []
	for dir, dirs, files in os.walk('zim'):
		if '__init__.py' in files:
			package = '.'.join(dir.split(os.sep))
			packages.append(package)
	#~ print('Pakages: ', packages)
	return packages


def get_mopath(pofile):
	# Function to determine right locale path for a .po file
	lang = os.path.basename(pofile)[:-3] # len('.po') == 3
	modir = os.path.join(LOCALE_FOLDER, lang, 'LC_MESSAGES')
	mofile = os.path.join(modir, 'zim.mo')
	return modir, mofile


def include_file(file):
	# Check to exclude hidden and temp files
	if file.startswith('.'):
		return False
	else:
		for ext in ('~', '.bak', '.swp', '.pyc'):
			if file.endswith(ext):
				return False
	return True


def collect_data_files():
	# Search for data files to be installed in share/
	data_files = [
		('share/man/man1', ['man/zim.1']),
		('share/applications', ['xdg/org.zim_wiki.Zim.desktop']),
		('share/mime/packages', ['xdg/org.zim_wiki.Zim.xml']),
		('share/metainfo', ['xdg/org.zim_wiki.Zim.appdata.xml']),
	]

	# xdg/hicolor -> PREFIX/share/icons/hicolor
	for dir, dirs, files in os.walk('xdg/hicolor'):
		if files:
			target = os.path.join('share', 'icons', dir[4:])
			files = [os.path.join(dir, f) for f in files]
			data_files.append((target, files))

	# mono icons -> PREFIX/share/icons/ubuntu-mono-light | -dark
	for theme in ('ubuntu-mono-light', 'ubuntu-mono-dark'):
		file = os.path.join('icons', theme, 'zim-panel.svg')
		target = os.path.join('share', 'icons', theme, 'apps', '22')
		data_files.append((target, [file]))

	# data -> PREFIX/share/zim
	for dir, dirs, files in os.walk('data'):
		if '.zim' in dirs:
			dirs.remove('.zim')
		target = os.path.join('share', 'zim', dir[5:])
		if files:
			files = list(filter(include_file, files))
			files = [os.path.join(dir, f) for f in files]
			data_files.append((target, files))

	# .po files -> PREFIX/share/locale/..
	for pofile in [f for f in os.listdir(PO_FOLDER) if f.endswith('.po')]:
		pofile = os.path.join(PO_FOLDER, pofile)
		modir, mofile = get_mopath(pofile)
		target = os.path.join('share', modir)
		data_files.append((target, [mofile]))

	#~ import pprint
	#~ print('Data files: ')
	#~ pprint.pprint(data_files)
	return data_files


def fix_dist():
	# Generate man page
	makeman.make()

	# Add the changelog to the manual
	# print('copying CHANGELOG.txt -> data/manual/Changelog.txt')
	# shutil.copy('CHANGELOG.txt', 'data/manual/Changelog.txt')

	# Copy the zim icons a couple of times
	# Paths for mimeicons taken from xdg-icon-resource
	# xdg-icon-resource installs:
	# /usr/local/share/icons/hicolor/.../mimetypes/application-x-zim-notebook.png
	# /usr/local/share/icons/hicolor/.../apps/org.zim_wiki.Zim.png

	if os.path.exists('xdg/hicolor'):
		shutil.rmtree('xdg/hicolor')
	os.makedirs('xdg/hicolor/scalable/apps')
	os.makedirs('xdg/hicolor/scalable/mimetypes')
	for name in (
		'apps/org.zim_wiki.Zim.svg',
		'mimetypes/application-x-zim-notebook.svg'
	):
		shutil.copy('icons/zim48.svg', 'xdg/hicolor/scalable/' + name)
	for size in ('16', '22', '24', '32', '48'):
		dir = size + 'x' + size
		os.makedirs('xdg/hicolor/%s/apps' % dir)
		os.makedirs('xdg/hicolor/%s/mimetypes' % dir)
		for name in (
			'apps/org.zim_wiki.Zim.png',
			'mimetypes/application-x-zim-notebook.png'
		):
			shutil.copy('icons/zim%s.png' % size, 'xdg/hicolor/' + dir + '/' + name)


# Overloaded commands

class zim_sdist_class(sdist_class):
	# Command to build source distribution
	# make sure _version.py gets build and included

	def initialize_options(self):
		sdist_class.initialize_options(self)
		self.force_manifest = 1 # always re-generate MANIFEST

	def run(self):
		fix_dist()
		sdist_class.run(self)


class zim_build_trans_class(Command):
	# Compile mo files

	description = 'Build translation files'
	user_options = []

	def initialize_options(self):
		pass

	def finalize_options(self):
		pass

	def run(self):
		for pofile in [f for f in os.listdir(PO_FOLDER) if f.endswith('.po')]:
			pofile = os.path.join(PO_FOLDER, pofile)
			modir, mofile = get_mopath(pofile)

			if not os.path.isdir(modir):
				os.makedirs(modir)

			if not os.path.isfile(mofile) or newer(pofile, mofile):
				print('compiling %s' % mofile)
				msgfmt.MESSAGES.clear() # prevent "spill over" between translations - see github #664
				msgfmt.make(pofile, mofile)
			else:
				#~ print('skipping %s - up to date' % mofile)
				pass


class zim_build_class(build_class):
	# Generate _version.py etc. and call build_trans as a subcommand
	# And put list of default plugins in zim/plugins/__init__.py

	sub_commands = build_class.sub_commands + [('build_trans', None)]

	def run(self):
		fix_dist()
		build_class.run(self)

		# Adjust bin/zim.py -> bin/zim
		if os.name == 'posix' and not self.dry_run:
			for script in find_scripts_in_build_dir():
				if script.endswith('.py'):
					print('renaming %s to %s' % (script, script[:-3]))
					os.rename(script, script[:-3]) # len('.py') == 3

		## Set default plugins
		plugins = []
		for name in os.listdir('./zim/plugins'):
			if name.startswith('_') or name == 'base':
				continue
			elif '.' in name:
				if name.endswith('.py'):
					name, x = name.rsplit('.', 1)
					plugins.append(name)
				else:
					continue
			else:
				plugins.append(name)
		assert len(plugins) > 20, 'Did not find plugins'


		file = os.path.join(self.build_lib, 'zim', 'plugins', '__init__.py')
		print('Setting plugin list in %s' % file)
		assert os.path.isfile(file)
		fh = open(file)
		lines = fh.readlines()
		fh.read()

		for i, line in enumerate(lines):
			if line.startswith('\t\tplugins = set('):
				lines[i] = '\t\tplugins = set(%r) # DEFAULT PLUGINS COMPILED IN BY SETUP.PY\n' % sorted(plugins)
				break
		else:
			assert False, 'Missed line for plugin list'

		fh = open(file, 'w')
		fh.writelines(lines)
		fh.close()



class zim_install_class(install_class):

	def run(self):
		install_class.run(self)
		mimedir = os.path.join(self.install_data, 'share', 'mime')
		print(
			'To register zim with the desktop environment, please run\n'
			'the following two commands:\n'
			'* update-desktop-database\n'
			'* update-mime-database %s\n' % mimedir
		)


# Distutils parameters, and main function

scripts = ['zim.py']

if py2exe:
	py2exeoptions = {
		'windows': [{
			'script': 'zim.py',
			'icon_resources': [(1, 'icons/zim.ico')]
				# Windows 16x16, 32x32, and 48x48 icon based on PNG
		}],
		'zipfile': None,
		'options': {
			'py2exe': {
				'compressed': 1,
				'optimize': 2,
				'ascii': 1,
				'bundle_files': 3,
				'packages': ['encodings', 'cairo', 'atk', 'pangocairo', 'zim'],
				'dll_excludes': {
					'DNSAPI.DLL'
				},
				'excludes': ['Tkconstants', 'Tkinter', 'tcl']
			}
		}
	}
else:
	py2exeoptions = {}

if __name__ == '__main__':
	setup(
		# wire overload commands
		cmdclass = {
			'sdist': zim_sdist_class,
			'build': zim_build_class,
			'build_trans': zim_build_trans_class,
			'install': zim_install_class,
		},

		# provide package properties
		name = 'zim',
		version = __version__,
		description = 'Zim desktop wiki',
		author = 'Jaap Karssenberg',
		author_email = 'jaap.karssenberg@gmail.com',
		license = 'GPL v2+',
		url = __url__,
		scripts = scripts,
		packages = collect_packages(),
		data_files = collect_data_files(),
		requires = ['gi', 'xdg'],

		**py2exeoptions
	)
